use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Perl/Critic/PJCJ.pm',
    'lib/Perl/Critic/Policy/CodeLayout/LimitLineLength.pm',
    'lib/Perl/Critic/Policy/ValuesAndExpressions/UseConsistentQuoting.pm',
    'lib/Perl/Critic/Utils/SourceLocation.pm',
    't/00-compile.t',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/CodeLayout/LimitLineLength/basic.t',
    't/CodeLayout/LimitLineLength/configuration.t',
    't/CodeLayout/LimitLineLength/line_number_reporting.t',
    't/Utils/SourceLocation/basic.t',
    't/ValuesAndExpressions/UseConsistentQuoting/basic.t',
    't/ValuesAndExpressions/UseConsistentQuoting/delimiter_optimisation.t',
    't/ValuesAndExpressions/UseConsistentQuoting/double_quotes.t',
    't/ValuesAndExpressions/UseConsistentQuoting/escape_sequences.t',
    't/ValuesAndExpressions/UseConsistentQuoting/messages.t',
    't/ValuesAndExpressions/UseConsistentQuoting/newlines.t',
    't/ValuesAndExpressions/UseConsistentQuoting/quote_like_operators.t',
    't/ValuesAndExpressions/UseConsistentQuoting/quote_operators.t',
    't/ValuesAndExpressions/UseConsistentQuoting/simple_quote_operators.t',
    't/ValuesAndExpressions/UseConsistentQuoting/single_quotes.t',
    't/ValuesAndExpressions/UseConsistentQuoting/use_statement_quote_types.t',
    't/lib/ViolationFinder.pm'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
